(ns com.aurellem.gb.stats
  (:use (com.aurellem.gb gb-driver util constants))
  (:import [com.aurellem.gb.gb_driver SaveState]))

(def pokemon-1-stats-start-address 0xD18B)

(defn pokemon-stats-address [poke-num]
  (+ pokemon-1-stats-start-address
     (* poke-num pokemon-record-width)))

(def stats-record-size 11)

(def pokemon-1-current-hp-address 0xD16B)

(defn current-hp-address [poke-num]
  (+ pokemon-1-current-hp-address
     (* pokemon-record-width poke-num)))

(defn read-current-hp
  ([^SaveState state poke-num]
     (let [mem (memory state)
           start (current-hp-address poke-num)
           hp-h (aget mem start)
           hp-l (aget mem (inc start))]
       (glue-bytes hp-h hp-l)))
  ([poke-num]
     (read-current-hp @current-state poke-num)))

(defn set-current-hp
  ([^SaveState state poke-num new-hp]
     (set-memory-range state (current-hp-address poke-num)
                 (disect-bytes-2 new-hp)))
  ([poke-num new-hp]
     (set-current-hp @current-state poke-num new-hp)))

(defn read-stats
  ([^SaveState state poke-num]
     (let [start (pokemon-stats-address poke-num)
           [level
            hp-h
            hp-l
            attack-h
            attack-l
            defense-h
            defense-l
            speed-h
            speed-l
            special-h
            special-l]
           (subvec (vec (memory state))
                   start (+ start stats-record-size ))]
       {:level level
        :current-hp (read-current-hp state poke-num)
        :hp (glue-bytes hp-h hp-l)
        :attack (glue-bytes attack-h attack-l)
        :defense (glue-bytes defense-h defense-l)
        :speed (glue-bytes speed-h speed-l)
        :special (glue-bytes special-h special-l)}))
  ([poke-num]
     (read-stats @current-state poke-num)))
          
(defn give-stats
  ([^SaveState state poke-num new-stats]
     (let [new-stats* (merge (read-stats state poke-num)
                             new-stats)
           raw-stats
           (flatten
            [(:level new-stats*)
             (disect-bytes-2 (:hp new-stats*))
             (disect-bytes-2 (:attack new-stats*))
             (disect-bytes-2 (:defense new-stats*))
             (disect-bytes-2 (:speed new-stats*))
             (disect-bytes-2 (:special new-stats*))])]
       (set-current-hp 
        (set-memory-range state (pokemon-stats-address poke-num)
                          raw-stats)
        poke-num
        (:current-hp new-stats*))))
  ([poke-num new-stats]
     (give-stats @current-state poke-num new-stats)))


       
     